@@ -16,7 +16,7 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import re, time, stat, unittest, os.path, struct, types, fcntl, shelve, marshal
+import re, time, stat, unittest, os.path, struct, types, shelve, marshal
 from StringIO import StringIO
 from pygopherd import zipfile
 
