@@ -5,8 +5,7 @@
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
+#    the Free Software Foundation; version 2 of the License.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,13 +19,68 @@
 import SocketServer
 import re
 import os, stat, os.path, mimetypes
-from pygopherd import protocols, handlers, gopherentry
+from pygopherd import protocols, gopherentry
 
 rootpath = None
 
+class VFS_Real:
+    def __init__(self, config, chain = None):
+        """This implementation does not chain."""
+        self.config = config
+    
+    def iswritable(self, selector):
+        return 1
+
+    def unlink(self, selector):
+        os.unlink(self.getfspath(selector))
+
+    def stat(self, selector):
+        return os.stat(self.getfspath(selector))
+
+    def isdir(self, selector):
+        return os.path.isdir(self.getfspath(selector))
+
+    def isfile(self, selector):
+        return os.path.isfile(self.getfspath(selector))
+
+    def exists(self, selector):
+        return os.path.exists(self.getfspath(selector))
+
+    def open(self, selector, *args, **kwargs):
+        return apply(open, (self.getfspath(selector),) + args, kwargs)
+
+    def listdir(self, selector):
+        return os.listdir(self.getfspath(selector))
+
+    def getrootpath(self):
+        global rootpath
+        if not rootpath:
+            rootpath = self.config.get("pygopherd", "root")
+        return rootpath
+
+    def getfspath(self, selector):
+        """Gets the filesystem path corresponding to the selector."""
+
+        fspath = self.getrootpath() + selector
+        # Strip off trailing slash.
+        if fspath[-1] == '/':
+            fspath = fspath[0:-1]
+
+        return fspath
+
+    def copyto(self, name, fd):
+        rfile = self.open(name, 'rb')
+        while 1:
+            data = rfile.read(4096)
+            if not len(data):
+                break
+            fd.write(data)
+        rfile.close
+
 class BaseHandler:
     """Skeleton handler -- includes commonly-used routines."""
-    def __init__(self, selector, searchrequest, protocol, config, statresult):
+    def __init__(self, selector, searchrequest, protocol, config, statresult,
+                 vfs = None):
         """Parameters are:
         selector -- requested selector.  The selector must always start
         with a slash and never end with a slash UNLESS it is a one-char
@@ -42,6 +96,10 @@ class BaseHandler:
         self.fspath = None
         self.entry = None
         self.searchrequest = searchrequest
+        if not vfs:
+            self.vfs = VFS_Real(self.config)
+        else:
+            self.vfs = vfs
 
     def isrequestforme(self):
         """Called by multiplexers or other handlers.  The default
@@ -79,31 +137,36 @@ class BaseHandler:
             self.entry = gopherentry.GopherEntry(self.selector, self.config)
         return self.entry
 
-    def getrootpath(self):
-        """Gets the root path."""
-        global rootpath
-        if not rootpath:
-            rootpath = self.config.get("pygopherd", "root")
-        return rootpath
-
     def getfspath(self):
-        """Gets the filesystem path corresponding to the selector."""
-        if self.fspath:
-            return self.fspath
-
-        self.fspath = self.getrootpath() + self.getselector()
-        # Strip off trailing slash.
-        if self.fspath[-1] == '/':
-            self.fspath = self.fspath[0:-1]
-
+        if not self.fspath:
+            self.fspath = self.vfs.getfspath(self.getselector())
         return self.fspath
 
+    def getselector(self):
+        """Returns the selector we are handling."""
+        return self.selector
+
+    def gethandler(self):
+        """Returns the handler to use to process this request.  For all
+        but special cases (rewriting handleres, for instance), this should
+        return self."""
+        return self
+
+    ## The next three are the publically-exposed interface -- the ones
+    ## called by things other than handlers.
+
     def prepare(self):
         """Prepares for a write.  Ie, opens a file.  This is
         used so that the protocols can try to detect an error before
         transmitting a result.  Must always be called before write."""
         pass
 
+    def isdir(self):
+        """Returns true if this handler is handling a directory; false
+        otherwise.  Not valid unless prepare has been called."""
+
+        return 0
+    
     def write(self, wfile):
         """Writes out the request if isdir() returns false.  You should
         NOT call write if isdir() returns true!  Should be overridden
@@ -111,16 +174,6 @@ class BaseHandler:
         if self.isdir():
             raise Exception, "Attempt to use write for a directory"
 
-    def getselector(self):
-        """Returns the selector we are handling."""
-        return self.selector
-
-    def isdir(self):
-        """Returns true if this handler is handling a directory; false
-        otherwise.  Not valid unless prepare has been called."""
-
-        return 0
-    
     def getdirlist(self):
         """Returns a list-like object (list, iterator, tuple, generator, etc)
         that contains as its elements the gopherentry objects corresponding
