@@ -5,8 +5,7 @@
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
+#    the Free Software Foundation; version 2 of the License.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +21,7 @@ import HTMLParser
 import SocketServer
 import re
 import os, stat, os.path, mimetypes
-from pygopherd import protocols, gopherentry, handlers
+from pygopherd import protocols, gopherentry
 from pygopherd.gopherentry import GopherEntry
 import htmlentitydefs
 from stat import *
@@ -71,7 +70,7 @@ class HTMLFileTitleHandler(FileHandler):
         # Start with the entry from the parent.
         entry = FileHandler.getentry(self)
         parser = HTMLTitleParser()
-        file = open(self.getfspath(), "rt")
+        file = self.vfs.open(self.getselector(), "rt")
         try:
             while not parser.gotcompletetitle:
                 line = file.readline()
